@@ -1,25 +1,30 @@
 /*
  * libpri: An implementation of Primary Rate ISDN
  *
- * Written by Mark Spencer <markster@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
  *
- * Copyright (C) 2001, Linux Support Services, Inc.
+ * Copyright (C) 2001, Digium, Inc.
  * All Rights Reserved.
+ */
+
+/*
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2 as published by the
+ * Free Software Foundation. See the LICENSE file included with
+ * this program for more details.
  *
+ * In addition, when this program is distributed with Asterisk in
+ * any form that would qualify as a 'combined work' or as a
+ * 'derivative work' (but not mere aggregation), you can redistribute
+ * and/or modify the combination under the terms of the license
+ * provided with that copy of Asterisk, instead of the license
+ * terms granted here.
  */
  
 #ifndef _LIBPRI_H
@@ -59,23 +64,23 @@
 
 /* PRI D-Channel Events */
 #define PRI_EVENT_DCHAN_UP		 1	/* D-channel is up */
-#define PRI_EVENT_DCHAN_DOWN 	 2	/* D-channel is down */
+#define PRI_EVENT_DCHAN_DOWN	 2	/* D-channel is down */
 #define PRI_EVENT_RESTART		 3	/* B-channel is restarted */
 #define PRI_EVENT_CONFIG_ERR 	 4	/* Configuration Error Detected */
-#define PRI_EVENT_RING			 5	/* Incoming call */
-#define PRI_EVENT_HANGUP		 6	/* Call got hung up */
-#define PRI_EVENT_RINGING		 7	/* Call is ringing (alerting) */
-#define PRI_EVENT_ANSWER		 8	/* Call has been answered */
+#define PRI_EVENT_RING			 5	/* Incoming call (SETUP) */
+#define PRI_EVENT_HANGUP		 6	/* Call got hung up (RELEASE/RELEASE_COMPLETE/other) */
+#define PRI_EVENT_RINGING		 7	/* Call is ringing (ALERTING) */
+#define PRI_EVENT_ANSWER		 8	/* Call has been answered (CONNECT) */
 #define PRI_EVENT_HANGUP_ACK	 9	/* Call hangup has been acknowledged */
-#define PRI_EVENT_RESTART_ACK	10	/* Restart complete on a given channel */
+#define PRI_EVENT_RESTART_ACK	10	/* Restart complete on a given channel (RESTART_ACKNOWLEDGE) */
 #define PRI_EVENT_FACNAME		11	/* Caller*ID Name received on Facility */
-#define PRI_EVENT_INFO_RECEIVED 12	/* Additional info (keypad) received */
-#define PRI_EVENT_PROCEEDING	13	/* When we get CALL_PROCEEDING or PROGRESS */
+#define PRI_EVENT_INFO_RECEIVED 12	/* Additional info (digits) received (INFORMATION) */
+#define PRI_EVENT_PROCEEDING	13	/* When we get CALL_PROCEEDING */
 #define PRI_EVENT_SETUP_ACK		14	/* When we get SETUP_ACKNOWLEDGE */
-#define PRI_EVENT_HANGUP_REQ	15	/* Requesting the higher layer to hangup */
-#define PRI_EVENT_NOTIFY		16	/* Notification received */
-#define PRI_EVENT_PROGRESS		17	/* When we get CALL_PROCEEDING or PROGRESS */
-#define PRI_EVENT_KEYPAD_DIGIT		18	/* When we receive during ACTIVE state */
+#define PRI_EVENT_HANGUP_REQ	15	/* Requesting the higher layer to hangup (DISCONNECT) */
+#define PRI_EVENT_NOTIFY		16	/* Notification received (NOTIFY) */
+#define PRI_EVENT_PROGRESS		17	/* When we get PROGRESS */
+#define PRI_EVENT_KEYPAD_DIGIT	18	/* When we receive during ACTIVE state (INFORMATION) */
 
 /* Simple states */
 #define PRI_STATE_DOWN		0
@@ -207,6 +212,52 @@
 #define PRI_LAYER_1_V120_RATE_ADAPT	0x28
 #define PRI_LAYER_1_X31_RATE_ADAPT	0x29
 
+
+/* Intermediate rates for V.110 */
+#define PRI_INT_RATE_8K			1
+#define PRI_INT_RATE_16K		2
+#define PRI_INT_RATE_32K		3
+
+
+/* Rate adaption for bottom 5 bits of rateadaption */
+#define PRI_RATE_USER_RATE_MASK		0x1F
+#define PRI_RATE_ADAPT_UNSPEC		0x00
+#define PRI_RATE_ADAPT_0K6		0x01
+#define PRI_RATE_ADAPT_1K2		0x02
+#define PRI_RATE_ADAPT_2K4		0x03
+#define PRI_RATE_ADAPT_3K6		0x04
+#define PRI_RATE_ADAPT_4K8		0x05
+#define PRI_RATE_ADAPT_7K2		0x06
+#define PRI_RATE_ADAPT_8K		0x07
+#define PRI_RATE_ADAPT_9K6		0x08
+#define PRI_RATE_ADAPT_14K4		0x09
+#define PRI_RATE_ADAPT_16K		0x0A
+#define PRI_RATE_ADAPT_19K2		0x0B
+#define PRI_RATE_ADAPT_32K		0x0C
+#define PRI_RATE_ADAPT_38K4		0x0D
+#define PRI_RATE_ADAPT_48K		0x0E
+#define PRI_RATE_ADAPT_56K		0x0F
+#define PRI_RATE_ADAPT_57K6		0x12
+#define PRI_RATE_ADAPT_28K8		0x13
+#define PRI_RATE_ADAPT_24K		0x14
+#define PRI_RATE_ADAPT_0K1345		0x15
+#define PRI_RATE_ADAPT_0K1		0x16
+#define PRI_RATE_ADAPT_0K075_1K2	0x17
+#define PRI_RATE_ADAPT_1K2_0K075	0x18
+#define PRI_RATE_ADAPT_0K05		0x19
+#define PRI_RATE_ADAPT_0K075		0x1A
+#define PRI_RATE_ADAPT_0K110		0x1B
+#define PRI_RATE_ADAPT_0K150		0x1C
+#define PRI_RATE_ADAPT_0K200		0x1D
+#define PRI_RATE_ADAPT_0K300		0x1E
+#define PRI_RATE_ADAPT_12K		0x1F
+
+/* in-band negotiation flag for rateadaption bit 5 */
+#define PRI_RATE_ADAPT_NEGOTIATION_POSS	0x20
+
+/* async flag for rateadaption bit 6 */
+#define PRI_RATE_ADAPT_ASYNC		0x40
+
 /* Notifications */
 #define PRI_NOTIFY_USER_SUSPENDED		0x00	/* User suspended */
 #define PRI_NOTIFY_USER_RESUMED			0x01	/* User resumed */
@@ -296,6 +347,8 @@ typedef struct pri_event_facname {
 	int channel;
 	int cref;
 	q931_call *call;
+	int callingpres;			/* Presentation of Calling CallerID */
+	int callingplan;			/* Dialing plan of Calling entity */
 } pri_event_facname;
 
 #define PRI_CALLINGPLANANI
@@ -405,77 +458,78 @@ typedef int (*pri_io_cb)(struct pri *pri, void *buf, int buflen);
    channel operating in HDLC mode with FCS computed by the fd's driver.  Also it
    must be NON-BLOCKING! Frames received on the fd should include FCS.  Nodetype 
    must be one of PRI_NETWORK or PRI_CPE.  switchtype should be PRI_SWITCH_* */
-extern struct pri *pri_new(int fd, int nodetype, int switchtype);
+struct pri *pri_new(int fd, int nodetype, int switchtype);
+struct pri *pri_new_bri(int fd, int ptpmode, int nodetype, int switchtype);
 
 /* Create D-channel just as above with user defined I/O callbacks and data */
-extern struct pri *pri_new_cb(int fd, int nodetype, int switchtype, pri_io_cb io_read, pri_io_cb io_write, void *userdata);
+struct pri *pri_new_cb(int fd, int nodetype, int switchtype, pri_io_cb io_read, pri_io_cb io_write, void *userdata);
 
 /* Retrieve the user data associated with the D channel */
-extern void *pri_get_userdata(struct pri *pri);
+void *pri_get_userdata(struct pri *pri);
 
 /* Set the user data associated with the D channel */
-extern void pri_set_userdata(struct pri *pri, void *userdata);
+void pri_set_userdata(struct pri *pri, void *userdata);
 
 /* Set Network Specific Facility for PRI */
-extern void pri_set_nsf(struct pri *pri, int nsf);
+void pri_set_nsf(struct pri *pri, int nsf);
 
 /* Set debug parameters on PRI -- see above debug definitions */
-extern void pri_set_debug(struct pri *pri, int debug);
+void pri_set_debug(struct pri *pri, int debug);
 
 /* Get debug parameters on PRI -- see above debug definitions */
-extern int pri_get_debug(struct pri *pri);
+int pri_get_debug(struct pri *pri);
 
 #define PRI_FACILITY_ENABLE
 /* Enable transmission support of Facility IEs on the pri */
-extern void pri_facility_enable(struct pri *pri);
+void pri_facility_enable(struct pri *pri);
 
 /* Run PRI on the given D-channel, taking care of any events that
    need to be handled.  If block is set, it will block until an event
    occurs which needs to be handled */
-extern pri_event *pri_dchannel_run(struct pri *pri, int block);
+pri_event *pri_dchannel_run(struct pri *pri, int block);
 
 /* Check for an outstanding event on the PRI */
 pri_event *pri_check_event(struct pri *pri);
 
 /* Give a name to a given event ID */
-extern char *pri_event2str(int id);
+char *pri_event2str(int id);
 
 /* Give a name to a node type */
-extern char *pri_node2str(int id);
+char *pri_node2str(int id);
 
 /* Give a name to a switch type */
-extern char *pri_switch2str(int id);
+char *pri_switch2str(int id);
 
 /* Print an event */
-extern void pri_dump_event(struct pri *pri, pri_event *e);
+void pri_dump_event(struct pri *pri, pri_event *e);
 
 /* Turn presentation into a string */
-extern char *pri_pres2str(int pres);
+char *pri_pres2str(int pres);
 
 /* Turn numbering plan into a string */
-extern char *pri_plan2str(int plan);
+char *pri_plan2str(int plan);
 
 /* Turn cause into a string */
-extern char *pri_cause2str(int cause);
+char *pri_cause2str(int cause);
 
 /* Acknowledge a call and place it on the given channel.  Set info to non-zero if there
    is in-band data available on the channel */
-extern int pri_acknowledge(struct pri *pri, q931_call *call, int channel, int info);
+int pri_acknowledge(struct pri *pri, q931_call *call, int channel, int info);
 
 /* Send a digit in overlap mode */
-extern int pri_information(struct pri *pri, q931_call *call, char digit);
+int pri_information(struct pri *pri, q931_call *call, char digit);
 
 #define PRI_KEYPAD_FACILITY_TX
 /* Send a keypad facility string of digits */
-extern int pri_keypad_facility(struct pri *pri, q931_call *call, char *digits);
+int pri_keypad_facility(struct pri *pri, q931_call *call, char *digits);
 
 /* Answer the incomplete(call without called number) call on the given channel.
    Set non-isdn to non-zero if you are not connecting to ISDN equipment */
-extern int pri_need_more_info(struct pri *pri, q931_call *call, int channel, int nonisdn);
+int pri_need_more_info(struct pri *pri, q931_call *call, int channel, int nonisdn);
 
 /* Answer the call on the given channel (ignored if you called acknowledge already).
    Set non-isdn to non-zero if you are not connecting to ISDN equipment */
-extern int pri_answer(struct pri *pri, q931_call *call, int channel, int nonisdn);
+int pri_answer(struct pri *pri, q931_call *call, int channel, int nonisdn);
 
 /* Set CRV reference for GR-303 calls */
 
@@ -492,59 +546,60 @@ extern int pri_answer(struct pri *pri, q931_call *call, int channel, int nonisdn
 
 /* Hangup a call */
 #define PRI_HANGUP
-extern int pri_hangup(struct pri *pri, q931_call *call, int cause);
+int pri_hangup(struct pri *pri, q931_call *call, int cause);
 
 #define PRI_DESTROYCALL
-extern void pri_destroycall(struct pri *pri, q931_call *call);
+void pri_destroycall(struct pri *pri, q931_call *call);
 
 #define PRI_RESTART
-extern int pri_restart(struct pri *pri);
+int pri_restart(struct pri *pri);
 
-extern int pri_reset(struct pri *pri, int channel);
+int pri_reset(struct pri *pri, int channel);
 
 /* Create a new call */
-extern q931_call *pri_new_call(struct pri *pri);
+q931_call *pri_new_call(struct pri *pri);
 
 /* Retrieve CRV reference for GR-303 calls.  Returns >0 on success. */
-extern int pri_get_crv(struct pri *pri, q931_call *call, int *callmode);
+int pri_get_crv(struct pri *pri, q931_call *call, int *callmode);
 
 /* Retrieve CRV reference for GR-303 calls.  CRV must be >0, call mode should be 0 */
-extern int pri_set_crv(struct pri *pri, q931_call *call, int crv, int callmode);
+int pri_set_crv(struct pri *pri, q931_call *call, int crv, int callmode);
 
 /* How long until you need to poll for a new event */
-extern struct timeval *pri_schedule_next(struct pri *pri);
+struct timeval *pri_schedule_next(struct pri *pri);
 
 /* Run any pending schedule events */
 extern pri_event *pri_schedule_run(struct pri *pri);
+extern pri_event *pri_schedule_run_tv(struct pri *pri, const struct timeval *now);
 
-extern int pri_call(struct pri *pri, q931_call *c, int transmode, int channel,
+int pri_call(struct pri *pri, q931_call *c, int transmode, int channel,
    int exclusive, int nonisdn, char *caller, int callerplan, char *callername, int callerpres,
 	 char *called,int calledplan, int ulayer1);
 
-extern struct pri_sr *pri_sr_new(void);
-extern void pri_sr_free(struct pri_sr *sr);
+struct pri_sr *pri_sr_new(void);
+void pri_sr_free(struct pri_sr *sr);
 
-extern int pri_sr_set_channel(struct pri_sr *sr, int channel, int exclusive, int nonisdn);
-extern int pri_sr_set_bearer(struct pri_sr *sr, int transmode, int userl1);
-extern int pri_sr_set_called(struct pri_sr *sr, char *called, int calledplan, int complete);
-extern int pri_sr_set_caller(struct pri_sr *sr, char *caller, char *callername, int callerplan, int callerpres);
-extern int pri_sr_set_redirecting(struct pri_sr *sr, char *num, int plan, int pres, int reason);
+int pri_sr_set_channel(struct pri_sr *sr, int channel, int exclusive, int nonisdn);
+int pri_sr_set_bearer(struct pri_sr *sr, int transmode, int userl1);
+int pri_sr_set_called(struct pri_sr *sr, char *called, int calledplan, int complete);
+int pri_sr_set_caller(struct pri_sr *sr, char *caller, char *callername, int callerplan, int callerpres);
+int pri_sr_set_redirecting(struct pri_sr *sr, char *num, int plan, int pres, int reason);
 #define PRI_USER_USER_TX
 /* Set the user user field.  Warning!  don't send binary data accross this field */
-extern void pri_sr_set_useruser(struct pri_sr *sr, const char *userchars);
+void pri_sr_set_useruser(struct pri_sr *sr, const char *userchars);
 
-extern void pri_call_set_useruser(q931_call *sr, const char *userchars);
+void pri_call_set_useruser(q931_call *sr, const char *userchars);
 
-extern int pri_setup(struct pri *pri, q931_call *call, struct pri_sr *req);
+int pri_setup(struct pri *pri, q931_call *call, struct pri_sr *req);
 
 /* Set a call has a call indpendent signalling connection (i.e. no bchan) */
-extern int pri_sr_set_connection_call_independent(struct pri_sr *req);
+int pri_sr_set_connection_call_independent(struct pri_sr *req);
 
-/* Send an MWI indication to a remote location.  If activate is non zero, activates, if zero, decativates */
-extern int pri_mwi_activate(struct pri *pri, q931_call *c, char *caller, int callerplan, char *callername, int callerpres, char *called, int calledplan);
+/* Send an MWI indication to a remote location.  If activate is non zero, activates, if zero, deactivates */
+int pri_mwi_activate(struct pri *pri, q931_call *c, char *caller, int callerplan, char *callername, int callerpres, char *called, int calledplan);
 
 /* Send an MWI deactivate request to a remote location */
-extern int pri_mwi_deactivate(struct pri *pri, q931_call *c, char *caller, int callerplan, char *callername, int callerpres, char *called, int calledplan);
+int pri_mwi_deactivate(struct pri *pri, q931_call *c, char *caller, int callerplan, char *callername, int callerpres, char *called, int calledplan);
 
 #define PRI_2BCT
 /* Attempt to pass the channels back to the NET side if compatable and
@@ -553,30 +608,40 @@ int pri_channel_bridge(q931_call *call1, q931_call *call2);
 
 /* Override message and error stuff */
 #define PRI_NEW_SET_API
-extern void pri_set_message(void (*__pri_error)(struct pri *pri, char *));
-extern void pri_set_error(void (*__pri_error)(struct pri *pri, char *));
+void pri_set_message(void (*__pri_error)(struct pri *pri, char *));
+void pri_set_error(void (*__pri_error)(struct pri *pri, char *));
 
 /* Set overlap mode */
 #define PRI_SET_OVERLAPDIAL
-extern void pri_set_overlapdial(struct pri *pri,int state);
+void pri_set_overlapdial(struct pri *pri,int state);
+
+/* QSIG logical channel mapping option, do not skip channel 16 */
+#define PRI_SET_CHAN_MAPPING_LOGICAL
+void pri_set_chan_mapping_logical(struct pri *pri, int state);
 
 #define PRI_DUMP_INFO_STR
-extern char *pri_dump_info_str(struct pri *pri);
+char *pri_dump_info_str(struct pri *pri);
 
 /* Get file descriptor */
-extern int pri_fd(struct pri *pri);
+int pri_fd(struct pri *pri);
 
 #define PRI_PROGRESS
-/* Send call proceeding */
-extern int pri_progress(struct pri *pri, q931_call *c, int channel, int info);
+/* Send progress */
+int pri_progress(struct pri *pri, q931_call *c, int channel, int info);
+
+/* Send progress with cause IE */
+int pri_progress_with_cause(struct pri *pri, q931_call *c, int channel, int info, int cause);
 
 #define PRI_PROCEEDING_FULL
 /* Send call proceeding */
-extern int pri_proceeding(struct pri *pri, q931_call *c, int channel, int info);
+int pri_proceeding(struct pri *pri, q931_call *c, int channel, int info);
+
+/* Enable inband progress when a DISCONNECT is received */
+void pri_set_inbanddisconnect(struct pri *pri, unsigned int enable);
 
 /* Enslave a PRI to another, so they share the same call list
    (and maybe some timers) */
-extern void pri_enslave(struct pri *master, struct pri *slave);
+void pri_enslave(struct pri *master, struct pri *slave);
 
 #define PRI_GR303_SUPPORT
 #define PRI_ENSLAVE_SUPPORT
@@ -587,20 +652,22 @@ extern void pri_enslave(struct pri *master, struct pri *slave);
 #define PRI_ANI
 
 /* Send notification */
-extern int pri_notify(struct pri *pri, q931_call *c, int channel, int info);
+int pri_notify(struct pri *pri, q931_call *c, int channel, int info);
+
+int pri_callrerouting_facility(struct pri *pri, q931_call *call, const char *dest, const char* original, const char* reason);
 
 /* Get/Set PRI Timers  */
 #define PRI_GETSET_TIMERS
-extern int pri_set_timer(struct pri *pri, int timer, int value);
-extern int pri_get_timer(struct pri *pri, int timer);
-extern int pri_timer2idx(char *timer);
+int pri_set_timer(struct pri *pri, int timer, int value);
+int pri_get_timer(struct pri *pri, int timer);
+int pri_timer2idx(char *timer);
 
 #define PRI_MAX_TIMERS 32
 
 #define PRI_TIMER_N200	0	/* Maximum numer of q921 retransmissions */
 #define PRI_TIMER_N201	1	/* Maximum numer of octets in an information field */
 #define PRI_TIMER_N202	2	/* Maximum numer of transmissions of the TEI identity request message */
-#define PRI_TIMER_K	3	/* Maximum number of outstanding I-frames */
+#define PRI_TIMER_K		3	/* Maximum number of outstanding I-frames */
 
 #define PRI_TIMER_T200	4	/* time between SABME's */
 #define PRI_TIMER_T201	5	/* minimum time between retransmissions of the TEI Identity check messages */
@@ -628,4 +695,10 @@ extern int pri_timer2idx(char *timer);
 #define PRI_TIMER_T321	26
 #define PRI_TIMER_T322	27
 
+#define PRI_TIMER_TM20	28	/* maximum time avaiting XID response */
+#define PRI_TIMER_NM20	29	/* number of XID retransmits */
+
+/* Get PRI version */
+const char *pri_get_version(void);
+
 #endif
